package org.apache.jmeter;

import org.apache.jmeter.samplers.SampleListener;
import org.apache.jmeter.timers.*;
import org.apache.jmeter.controllers.*;
import org.apache.jmeter.visualizers.*;
import org.apache.jmeter.samplers.*;

import java.net.URL;
import java.io.IOException;

public class TextDriver implements Runnable {
    JMeterEngine engine;
    SamplerController sc;
    String[] thread_groups;
    int[] thread_counts;
    private static final String USAGE =
	"java org.apache.jmeter.TextDriver [-o output-file] url";
    OutputListener ol;

    public TextDriver(String outfile, String url, String propsFile) {
	JMeterUtils.getProperties(propsFile);
	sc = new SimpleHTTPController();
	ol = new OutputListener(outfile);
	engine = new JMeterEngine();

	((SimpleHTTPController)sc).setURL(url);

	engine.setSamplerController(sc);
	engine.setTimer(new GaussianRandomTimer());
	engine.addSampleListener(ol);

	thread_groups = new String[] {"Group 1"};
	thread_counts = new int[] {1};
    }

    public void run() {
	engine.start(thread_groups, thread_counts);
	while(true) {
	    try {
		Thread.currentThread().sleep(5000);
	    } catch (InterruptedException e) {}
	    try {
		ol.flush();
	    } catch (IOException e) {e.printStackTrace();}
	}
    }

    private class OutputListener implements SampleListener {
	FileVisualizer viz;
	private OutputListener(String file) {
	    viz = new FileVisualizer();
	    viz.setFile(file);
	    try {
		viz.open();
	    } catch (IOException e) {e.printStackTrace();}
	}

	public void flush() throws IOException {
	    viz.flush();
	}

	public void close() throws IOException {
	    viz.close();
	}

	public void sampleOccurred(SampleEvent e) {
	    viz.add(e.getResult());
	}

	public void sampleStarted(SampleEvent e) {}
	public void sampleStopped(SampleEvent e) {}
    }

    public static void main(String[] args)
	throws java.net.MalformedURLException
    {
	if (args.length < 1) {
	    System.err.println(USAGE);
	    System.exit(-1);
	}

	String url=args[args.length-1];
	String outfile = "jmeter.out";
	String propsFile = null;

	char opt=(char)-1;
	for(int i=0;i<(args.length-1);i++)
	    if (opt==(char)-1)  {
		if (args[i].charAt(0)=='-') {
		    opt=args[i].charAt(1);
		}
		else {
		    System.err.println("Illegal option "+args[i]);
		    System.exit(-1);
		}
	    } else {
		switch (opt) {
		case 'o':
		    outfile = args[i];
		    break;
		case 'p':
		    propsFile=args[i];
		    break;
		default:
		    System.err.println("Unknown option: "+opt);
		    System.exit(-1);
		}
		opt=(char)-1;
	    }

	TextDriver td = new TextDriver(outfile, url, propsFile);
	Thread t = new Thread(td);
	t.start();
    }
}
