package org.apache.jmeter.controllers;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

import org.apache.jmeter.samplers.CookieManager;
import org.apache.jmeter.samplers.Cookie;
import org.apache.jmeter.JMeterUtils;

/**
 * Allows the user to specify if she needs cookie services, and
 * give parameters for this service.
 */
public class CookiePanel extends JPanel implements ActionListener {
    /**
     * Allows GUI interface for
     */
    JButton specifyButton;
    JCheckBox cookieCheck;

    CookieManager manager = new CookieManager();

    /**
     * A table to show the authentication information
     */
    JTable cookieTable;

    /**
     *  Dialog to edit cookie entries
     */
    JDialog entryDialog;

    /**
     * Default constructor
     **/
    public CookiePanel() {
	EtchedBorder etched = new EtchedBorder();
	setBorder(new TitledBorder(etched, "Cookies"));
        setLayout(new FlowLayout(FlowLayout.LEFT));

	cookieCheck = new JCheckBox("Enable",true);
	cookieCheck.setActionCommand("cookie");
	cookieCheck.addActionListener(this);
	add(cookieCheck);

	specifyButton = new JButton("Config");
	specifyButton.setEnabled(true);
	specifyButton.setActionCommand("edit");
	specifyButton.addActionListener(this);
	add(specifyButton);
    }

    /**
     * Disable the panel
     **/
    public void disable() {
	cookieCheck.setEnabled(false);
	specifyButton.setEnabled(false);
    }

    /**
     * Enable the panel
     **/
    public void enable() {
	cookieCheck.setEnabled(true);
        if (cookieCheck.isSelected()) {
            specifyButton.setEnabled(true);
        } else {
            specifyButton.setEnabled(false);
        }
    }

    /**
     * Understands the following ActionCommands:
     * <DL>
     * <DT>cookie
     * <DD>Toggles the availability of this panel.
     * <DT>browse
     * <DD>Invokes the file browser to locate files.
     * </DL>
     */
    public void actionPerformed(ActionEvent e) {
	String command = e.getActionCommand();
	if (command.equals("cookie")) {
	    if (cookieCheck.isSelected()) {
                specifyButton.setEnabled(true);
            } else {
                specifyButton.setEnabled(false);
            }
	} else if (command.equals("edit")) {
              showEditDialog();
        }
    }
    /**
     * Gets a CookieManager to manage the file that is currently
     * selected.  Null is returned if no file is currently selected.
     * Null will also be returned if there is a problem reading the
     * file while getting the CookieManager.
     *@returns A CookieManager for the current file, or null
     */
    public CookieManager getCookieMgr() {
	return cookieCheck.isSelected() ? manager : null;
    }

    /**
     * Shows the main cookie configuration panel
     */
    public void showEditDialog() {
        if (entryDialog == null) {
            entryDialog = new JDialog();
            entryDialog.setSize(400,500);
            cookieTable = new JTable(manager);
            cookieTable.setCellSelectionEnabled(false);
            cookieTable.setRowSelectionAllowed(true);
            cookieTable.setColumnSelectionAllowed(false);
            cookieTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

            JPanel main = new JPanel();
            main.setLayout(new BorderLayout());

            JScrollPane scroller = new JScrollPane(cookieTable);
            JTableHeader tableHeader = cookieTable.getTableHeader();
            scroller.setColumnHeaderView(tableHeader);
            main.add(scroller);

            JButton add = new JButton("Add");
            JButton edit = new JButton("Edit");
            JButton delete = new JButton("Delete");
            JButton close = new JButton("Close");
            JButton load = new JButton("Load");
            JButton save = new JButton("Save");
            Dimension d = delete.getPreferredSize();
            add.setPreferredSize(d);
            edit.setPreferredSize(d);
            close.setPreferredSize(d);
            load.setPreferredSize(d);
            save.setPreferredSize(d);
            CookieUpdater updater = new CookieUpdater();
            add.addActionListener(updater);
            edit.addActionListener(updater);
            delete.addActionListener(new ActionListener() {
            		public void actionPerformed(ActionEvent e) {
 				int row = cookieTable.getSelectedRow();
				if (row >= 0) {
					manager.remove(row);
				}
			}
	    });
            close.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				entryDialog.setVisible(false);
			}
            });
            load.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					File tmp = JMeterUtils.promptToOpenFile()
							.getSelectedFile();
					if (tmp != null) {
						manager.addFile(tmp.getAbsolutePath());
					}
				} catch (IOException ex) {
					ex.printStackTrace();
				}
			}
            });

            save.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					File tmp = JMeterUtils.promptToOpenFile()
							.getSelectedFile();
					if (tmp != null) {
						manager.save(tmp.getAbsolutePath());
					}
				} catch (IOException ex) {
					ex.printStackTrace();
				}
			}
            });

            JPanel panel = new JPanel();
            GridBagLayout g = new GridBagLayout();
            panel.setLayout(g);
            GridBagConstraints c = new GridBagConstraints();
            c.fill = c.NONE;
            c.gridwidth = 1;
            c.gridheight = 1;
            c.gridx = 1;
            c.gridy = 1;
            g.setConstraints(add, c);
            panel.add(add);
            c.gridx = 2;
            c.gridy = 1;
            g.setConstraints(edit, c);
            panel.add(edit);
            c.gridx = 3;
            c.gridy = 1;
            g.setConstraints(delete, c);
            panel.add(delete);
            c.gridx = 1;
            c.gridy = 2;
            g.setConstraints(close, c);
            panel.add(close);
            c.gridx = 2;
            c.gridy = 2;
            g.setConstraints(load, c);
            panel.add(load);
            c.gridx = 3;
            c.gridy = 2;
            g.setConstraints(save, c);
            panel.add(save);
            main.add(panel, "South");
            entryDialog.getContentPane().add(main);
        }
        entryDialog.show();
    }
    /**
     * Updates a cookie record
     */
    class CookieUpdater implements ActionListener {
	int index;
        JTextField nameField = new JTextField(20);
        JTextField valueField = new JTextField(20);
        JTextField domainField = new JTextField(20);
        JTextField pathField = new JTextField(20);
        JCheckBox secureCheck = new JCheckBox("Secure");
        JTextField expiresField = new JTextField(20);
        JButton ok = new JButton("Ok");
        JButton cancel = new JButton("Cancel");

        JDialog updateDialog;

	public CookieUpdater() {
	}

        /**
         * returns the contructed panel containing the cookie record
         */
	public JPanel getPanel() {
	    JPanel main = new JPanel();
            GridBagLayout g = new GridBagLayout();

            main.setLayout(g);
            GridBagConstraints c = new GridBagConstraints();
            c.fill = c.BOTH;
            c.gridwidth = 1;
            c.gridheight = 1;
            JLabel nameLabel = new JLabel("Name:");
            c.gridx = 1;
            c.gridy = 1;
            g.setConstraints(nameLabel, c);
            main.add(nameLabel);
            JLabel valueLabel = new JLabel("Value:");
            c.gridx = 1;
            c.gridy = 2;
            g.setConstraints(valueLabel, c);
            main.add(valueLabel);
            JLabel domainLabel = new JLabel("Domain:");
            c.gridx = 1;
            c.gridy = 3;
            g.setConstraints(domainLabel, c);
            main.add(domainLabel);
            JLabel pathLabel = new JLabel("Path:");
            c.gridx = 1;
            c.gridy = 4;
            g.setConstraints(pathLabel, c);
            main.add(pathLabel);
            JLabel expiresLabel = new JLabel("Expiration:");
            c.gridx = 1;
            c.gridy = 6;
            g.setConstraints(expiresLabel, c);
            main.add(expiresLabel);

            c.gridwidth = 2;
            c.gridx = 1;
            c.gridy = 5;
            g.setConstraints(secureCheck, c);
            c.gridwidth = 1;
            main.add(secureCheck);

            c.gridx = 2;
            c.gridy = 1;
            g.setConstraints(nameField, c);
            main.add(nameField);
            c.gridx = 2;
            c.gridy = 2;
            g.setConstraints(valueField, c);
            main.add(valueField);
            c.gridx = 2;
            c.gridy = 3;
            g.setConstraints(domainField, c);
            main.add(domainField);
            c.gridx = 2;
            c.gridy = 4;
            g.setConstraints(pathField, c);
            main.add(pathField);
            c.gridx = 2;
            c.gridy = 6;
            g.setConstraints(expiresField, c);
            main.add(expiresField);

            JPanel buttons = new JPanel();
            ok.setPreferredSize(cancel.getPreferredSize());
            buttons.add(ok);
            buttons.add(cancel);
            c.gridwidth = 2;
            c.gridx = 1;
            c.gridy = 7;
            g.setConstraints(buttons, c);
            main.add(buttons);

            return main;
	}

	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		boolean valid = true;
		index = -1;
		if (command.equals("Edit")) {
                    index = cookieTable.getSelectedRow();
                    if (index < 0) {
                        valid = false;
                    } else {
                        Cookie c = manager.get(index);
                        nameField = new JTextField(c.getName(), 20);
                        valueField = new JTextField(c.getValue(), 20);
                        domainField = new JTextField(c.getDomain(), 20);
                        pathField = new JTextField(c.getPath(), 20);
                        secureCheck = new JCheckBox("Secure", c.getSecure());
                        expiresField = new JTextField(new Long(c.getExpires()).toString(), 20);
                        ok = new JButton("Ok");
                        cancel = new JButton("Cancel");
                    }
                } else if (command.equals("Add")) {
                    nameField = new JTextField(20);
                    valueField = new JTextField(20);
                    domainField = new JTextField(20);
                    pathField = new JTextField(20);
                    secureCheck = new JCheckBox("Secure");
                    expiresField = new JTextField(20);
                    ok = new JButton("Ok");
                    cancel = new JButton("Cancel");
                }
		if (valid) {
                    if (updateDialog != null) {
                        updateDialog.dispose();
                    }
                    updateDialog = new JDialog();
                    updateDialog.setSize(350, 300);

                    ok.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            int i = index;
                            Cookie c = new Cookie();
                            if (i >= 0) {
                                c = manager.get(index);
                            }
                            c.setName(nameField.getText());
                            c.setValue(valueField.getText());
                            c.setDomain(domainField.getText());
                            c.setPath(pathField.getText());
                            c.setSecure(secureCheck.isSelected());
                            try {
                                c.setExpires(Long.parseLong(expiresField.getText()));
                            } catch (NumberFormatException ex) {
                                c.setExpires(0);
                            }
                            if (i < 0) {
                                manager.add(c);
                            }
                            updateDialog.dispose();
                        }
                    });
                    cancel.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            updateDialog.dispose();
                        }
                    });
                    updateDialog.getContentPane().add(getPanel());
                    updateDialog.show();
		}
	}
    }
}
