/*
 * Copyright (c) 1998-99 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * 4. The names "Apache JMeter" and "Java Apache Project" must
 *    not be used to endorse or promote products derived from this software
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JMeter"
 *    nor may "Java Apache Project" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */
package org.apache.jmeter.controllers;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

import javax.swing.*;
import javax.swing.table.*;
import javax.swing.border.*;
import java.io.*;

import org.apache.jmeter.JMeterUtils;
import org.apache.jmeter.samplers.*;
import org.apache.jmeter.util.*;

import org.xml.sax.*;

/**
 * Provides common functionality that any HTTP based Test Sample
 * will need.  It is able to import and export itself into an
 * XML format for loading later.
 */
public class HTTPBaseTestSample implements TestSample, Saveable
{

    private static String CLASS_DESC_NAME = "WebPage Test Sample";

    /** Entries for this Sample to visit. */
    Entry[] entriesArray;
    /** All the Thread Groups in the system. */
    String[] allThreadGroups;
    /** The Container used to edit this Sample's parameters. */
    HTTPTestSampleGUI gui;
    String name;

    boolean saveEnabled=true;

    /********************************************************/

    /**
     * Default Constructor
     **/
    public HTTPBaseTestSample()
    {
	name = null;
    }

    public String getClassDescName()
    {
      return CLASS_DESC_NAME;
    }

    public Container getGUI()
    {
	if(gui == null)
	    gui = new HTTPTestSampleGUI();
	return gui;
    }

    public void setThreadGroups(String[] threadGroups)
    {
      allThreadGroups = threadGroups;
	int[] chosen = getThreadGroupIndices();
	((HTTPTestSampleGUI)getGUI()).setThreadGroups(threadGroups);
	((HTTPTestSampleGUI)getGUI()).setSelectedThreadIndices(chosen);
    }


    public String[] getThreadGroups()
    {
	if (allThreadGroups.length==1)
	    return allThreadGroups;
	else {
	    int[] indices=((HTTPTestSampleGUI)getGUI()).getSelectedThreadIndices();
	    String[] result = new String[indices.length];
	    for(int i=0;i<indices.length;i++)
		result[i]=allThreadGroups[indices[i]];
	    return result;
	}
    }

    public String getName() {
      name = ((HTTPTestSampleGUI)getGUI()).getName();
	if (name==null)
	    return ((HTTPTestSampleGUI)getGUI()).getUrl();
	return name;
    }

    public void setName(String name) {
	this.name=name;
    }

    /**
     * There are no such saveable objects that test sample needs or specifies,
     * but if and when they do, this method is waiting. (think config elements)
     */
    public void addSaveableObject(Saveable o)
    {
    }


    public void setUrl(String url) {
	((HTTPTestSampleGUI)getGUI()).setURL(url);
    }

    public void setMethod(String method) {
	if (method.equalsIgnoreCase("post"))
	    ((HTTPTestSampleGUI)getGUI()).setPost(true);
	else
	    ((HTTPTestSampleGUI)getGUI()).setGet(true);
    }

    public void setOnce(boolean once) {
	((HTTPTestSampleGUI)getGUI()).setOnce(once);
    }

    public void setArguments(String[][] args) {
	if (args== null || args.length==0)
	    ((HTTPTestSampleGUI)getGUI()).generateArgumentFields();
	else
	    ((HTTPTestSampleGUI)getGUI()).generateArgumentFields(args);
    }

    private Entry getEntry()
    {
	HTTPEntry e = new HTTPEntry();
	e.setPrimary(HTTPSampler.URL);
	e.putValue(HTTPSampler.URL,((HTTPTestSampleGUI)getGUI()).getUrl());
	e.putValue(HTTPSampler.GET,new Boolean(isGet()));
	e.putValue(HTTPSamplerController.ONCE,new Boolean(isOnce()));
        e.putValue(HTTPSamplerController.INTERLEAVE,new Boolean(isInterleave()));
	e.putValue(HTTPSampler.POST,new Boolean(isPost()));
	e.putValue(HTTPSamplerController.THREADGROUPS,getThreadGroups());
	e.putValue(HTTPSampler.ARGUMENTS,
		   ((HTTPTestSampleGUI)getGUI()).getArguments());
	return e;
    }

    public void reset()
    {
      entriesArray = null;
    }

    /**
    * Returns an array of Entry[] based on the info gathered from the user about
    * the TestSample.  A single TestSample may generate one or more Entry objects.
    * For this particular TestSample, the reason for this is user's may enter
    * more than one argument value for each argument name they generate.  This
    * method will create one entry for every possible combination of arguments
    * and their values.
    * @return Array of Entry objects
    **/
    public synchronized Entry[] getEntries()
    {
      if(entriesArray != null)
        return entriesArray;
      Entry rawEntry = getEntry();
      String[][] arguments = (String[][])rawEntry.getValue(HTTPSampler.ARGUMENTS);
      java.util.List entries = new LinkedList();

      /* Will hold number of argument values of each argument name */
      int[] numArgs = new int[arguments.length];
      /* Will save our place in the argument value list when we do recursion */
      int[] place = new int[numArgs.length];
      java.util.List argNames = new LinkedList();
      java.util.List argValueLists = new LinkedList();
      int y = 0;
      for(int x = 0;x < arguments.length;x++)
      {
        String[] vals;
        if(arguments[x].length == 2)
        {
          String rawValues = arguments[x][1];
          int index;
	  for(index = 0;(index<rawValues.length()&&rawValues.charAt(index) == '"');index++);
          if(index == rawValues.length())
            rawValues = "";
          else
            rawValues = rawValues.substring(index);
          index = rawValues.length()-1;
          while(index >= 0 && rawValues.charAt(index) == '"') index--;
          rawValues = rawValues.substring(0,index+1);
          vals = JMeterUtils.split(rawValues,"\",\"","");
        }
        else vals = new String[]{""};

        /* Have split argument names from their list of values, now add each to their
        respective List */
        argNames.add(arguments[x][0]);
        argValueLists.add(vals);

        /* Initializing */
        numArgs[y] = vals.length;
        place[y++] = 0;
      }
      if(arguments.length == 0)
      {
        entriesArray = new Entry[]{rawEntry};
        return entriesArray;
      }

      /* a small hack to start things out right */
      place[place.length-1] = -1;
      recurseParameters(0,place.length-1,place,numArgs,rawEntry,argNames,argValueLists,entries);
      if(entries.size() == 0)
      {
        entriesArray = new Entry[]{rawEntry};
        return entriesArray;
      }
      else
      {
        entriesArray = (Entry[])entries.toArray(new Entry[0]);
        return entriesArray;
      }
    }

    /**
    * A recursive method that will iterator through all possible combinations of
    * argument names and their values.  It will generate an Entry for each
    * combination and store it in the entries List object.
    **/
    private static void recurseParameters(int base,int start,int[] place,int[] lengths,Entry baseEntry,
            java.util.List argNames,java.util.List argValueLists,java.util.List entries)
    {

      /* Start at the end and work back, every time we reach the end of the value
      list, reset it to zero, and then move back one argument and bump the place
      holder by 1.  If we've gone past the beginning of the list of arguments,
      we're done */
      for(;start >= base && ++place[start] >= lengths[start];--start)
        place[start] = 0;
      if(start < base)
        return;
      else
        start = place.length-1;

      /* Make a copy with the base info */
      Entry entry = (Entry)baseEntry.clone();
      int z = 0;
      String[][] arguments = new String[argNames.size()][2];
      int count = 0;
      for(Iterator x = argNames.iterator(), y = argValueLists.iterator();
              x.hasNext() && y.hasNext();)
      {
        String name = (String)x.next();
        String[] vals = (String[])y.next();
        /* Using the value of the appropriate place holder, create the arg-value
        pair and add to the entry */
        arguments[count][0] = name;
        arguments[count++][1] = vals[place[z++]];

      }
      entry.putValue(HTTPSampler.ARGUMENTS,arguments);
      entries.add(entry);
      recurseParameters(base,start,place,lengths,baseEntry,argNames,argValueLists,entries);
    }

    private boolean isPost()
    {
	return ((HTTPTestSampleGUI)getGUI()).isPost();
    }

    private boolean isGet()
    {
	return ((HTTPTestSampleGUI)getGUI()).isGet();
    }

    private boolean isOnce()
    {
	return ((HTTPTestSampleGUI)getGUI()).isOnce();
    }

    private boolean isInterleave()
    {
      return ((HTTPTestSampleGUI)getGUI()).isInterleave();
    }

    private int[] getThreadGroupIndices()
    {
	return ((HTTPTestSampleGUI)getGUI()).getSelectedThreadIndices();
    }

    public void save(OutputStream output) {
	PrintWriter out = new PrintWriter(output);

	out.println("<?xml version=\"1.0\" ?>");
	String method;
	if (isPost())
	    method="POST";
	else
	    method="GET";
	out.println("<HTTPSample class=\""+this.getClass().getName()+
                    "\" once=\""+isOnce()+"\" method=\""+method+"\" url=\""+
		    ((HTTPTestSampleGUI)getGUI()).getUrl()+"\" name=\""+getName()+"\" >");
	String[][] args = ((HTTPTestSampleGUI)getGUI()).getArguments();

	out.println("<arguments>");
	for(int i=0;i<args.length;i++)
	    out.println("<argument name=\""+args[i][0]+"\">"+
			args[i][1]+"</argument>");
	out.println("</arguments>");
	out.println("</HTTPSample>");
	out.flush();
    }

  /**
   *  Gets the SAX document handler to read the xml that stores information
   *  necessary for the reconstruction of this type of object.
   */
  public HandlerBase getHandlerBase()
  {
    return (HandlerBase)JMeterUtils.getXMLDocumentHandler("httpsample");
  }

    /**@link dependency*//*#HTTPSampleDocumentHandler hnd;*/
  public void load(HandlerBase agent)
  {
    HTTPSampleDocumentHandler hnd;
	  try {
		  hnd = (HTTPSampleDocumentHandler)agent;
      if(hnd.getClassName().equals(this.getClass().getName()))
      {
		    HTTPTestSampleGUI tsg = ((HTTPTestSampleGUI)getGUI());
		    tsg.setURL(hnd.getURL());
		    if ("POST".equalsIgnoreCase(hnd.getMethod()))
			    tsg.setPost(true);
		    else
			    tsg.setGet(true);
		    tsg.setOnce(hnd.isOnce());
		    String[][] args = hnd.getArguments();
		    if (args==null || args.length==0)
			    tsg.generateArgumentFields();
		    else
			    tsg.generateArgumentFields(args);
        tsg.setSampleName(hnd.getName());
      }
      else
        JOptionPane.showMessageDialog(null,"Saved Test Sample doesn't match this class",
              "Error",JOptionPane.ERROR_MESSAGE);
	  } catch (Exception e) {
        JOptionPane.showMessageDialog(null,"Error occurred while parsing XML file",
              "Error",JOptionPane.ERROR_MESSAGE);
		    e.printStackTrace();
	  }
  }
}







