/*
 * Copyright (c) 1998-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * 4. The names "Apache JMeter" and "Java Apache Project" must
 *    not be used to endorse or promote products derived from this software
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JMeter"
 *    nor may "Java Apache Project" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */
package org.apache.jmeter.controllers;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

import javax.swing.*;
import javax.swing.table.*;
import javax.swing.border.*;
import java.io.*;

import org.apache.jmeter.JMeterUtils;
import org.apache.jmeter.samplers.*;
import org.xml.sax.*;
/**
 * The panel for setting the entry, it's parameters and any
 * arguments the entry might need.  This is the class that
 * is returned when the HTTPBaseTestSample is asked for it's
 * GUI.
 */
public class HTTPMultipartTestSampleGUI extends JPanel
{
    //Components at top
    BorderLayout topLevelLayout = new BorderLayout();
    JTextField urlField = new JTextField(45);
    JTextField nameField = new JTextField(25);
    JLabel threadGroupLabel = new JLabel();
    JList threadGroupField = new JList();
    JCheckBox getButton = new JCheckBox("GET", true);
    JCheckBox postButton = new JCheckBox("POST");
    JCheckBox onceOnly = new JCheckBox("Hit URL once only");
    JCheckBox interleave = new JCheckBox("Interleave");
    //JButton openButton = new JButton("Open");
    //JButton saveButton = new JButton("Save");
    JComboBox fileList;
    JButton browseButton;

    // Argument panel components
    private final String[] HEADERS = new String[] {"Name", "Value"};
    private final String ARGUMENTS = "httpbasetestsample.arguments";
    JTable argTable;
    ArgumentListener argListener;

    JDialog multValues;
    ListDialog multList;

    /**
     * Default constructor
     **/
    public HTTPMultipartTestSampleGUI()
    {
	Border title = new TitledBorder(new EtchedBorder(),
					"URL Test Parameters");

	threadGroupLabel.setText("Thread Groups");

	this.setLayout(topLevelLayout);
	this.setBorder(title);

	getButton.setActionCommand("get");
	postButton.setActionCommand("post");
	ButtonGroup postget = new ButtonGroup();
	postget.add(getButton);
	postget.add(postButton);
  argListener = new ArgumentListener();
	this.add(getURLPanel(), BorderLayout.NORTH);
	this.add(getArgListPanel(), BorderLayout.CENTER);


    }

    protected JPanel getURLPanel() {
	JPanel urlPanel = new JPanel();
	JLabel urlLabel = new JLabel("URL");

	urlPanel.setLayout(new GridBagLayout());
	GridBagConstraints gbc = new GridBagConstraints();
  gbc.insets = new Insets(2,2,2,2);
  gbc.gridx = 0;
  gbc.anchor = gbc.EAST;
  gbc.gridy = 0;
  urlPanel.add(new JLabel("Name"),gbc.clone());
  gbc.gridy++;
  gbc.fill = gbc.HORIZONTAL;
  gbc.weightx = 0.9;
  gbc.gridwidth=3;
  urlPanel.add(nameField,gbc.clone());
  gbc.weightx = 0;
  gbc.anchor = gbc.WEST;
  gbc.fill = gbc.NONE;
	gbc.gridx=0;
	gbc.gridy++;
  gbc.gridwidth=1;
	urlPanel.add(urlLabel, gbc.clone());
	gbc.gridy++;
	gbc.fill=gbc.HORIZONTAL;
	gbc.weightx=0.9;
	gbc.gridwidth=2;
	urlPanel.add(urlField, gbc.clone());
	gbc.weightx=0.1;
	gbc.fill=gbc.NONE;
	gbc.gridwidth=1;
	gbc.gridx+=2;
	urlPanel.add(postButton, gbc.clone());
	gbc.gridx++;
	urlPanel.add(getButton, gbc.clone());
	gbc.gridx=2;
	gbc.gridwidth=2;
	gbc.gridy++;
	urlPanel.add(onceOnly, gbc.clone());
        gbc.gridy++;
        urlPanel.add(interleave,gbc.clone());
  gbc.gridy--;
	gbc.gridx=0;
	gbc.gridwidth=1;
	urlPanel.add(threadGroupLabel, gbc.clone());
	gbc.gridy++;
	gbc.weighty = 1;
	gbc.fill=gbc.HORIZONTAL;
	threadGroupField.setBorder
	    (BorderFactory.createLineBorder(new Color(0,0,0),2));
	urlPanel.add(threadGroupField, gbc.clone());
  gbc.gridy++;
  gbc.gridx = 0;
  gbc.gridwidth=3;
  gbc.weighty=0;
  fileList = new JComboBox();
  fileList.setEditable(true);
  fileList.addActionListener(argListener);
  urlPanel.add(fileList,gbc.clone());

  browseButton = new JButton("Browse");
  browseButton.addActionListener(argListener);
  browseButton.setEnabled(true);
  gbc.gridx+=3;
  gbc.gridwidth=1;
  gbc.fill=gbc.NONE;
  urlPanel.add(browseButton,gbc.clone());


	return urlPanel;
    }

    /**
     * Gets the name the user has given for the test sample
     */
    public String getName()
    {
      String name = nameField.getText();
      if(!name.equals(""))
        return name;
      else return null;
    }

    public void setSampleName(String name)
    {
      System.out.println("Name = "+name);
      nameField.setText(name);
    }

    protected JPanel getArgListPanel() {
	Border argListBorder = new TitledBorder(new EtchedBorder(),
						"Argument List");

	//stuff for dialog
  multValues = new JDialog();
  multValues.setTitle("Add Argument");
	multValues.setSize(175,200);
	multList = new ListDialog(argListener);

	JPanel argListPanel = new JPanel();
	argListPanel.setLayout(new BorderLayout());

	argListPanel.setBorder(argListBorder);
	argTable = new JTable(1,2);
	//argTable.setCellEditor(new DefaultCellEditor(new JComboBox()));
	argTable.addKeyListener(argListener);
	argTable.addMouseListener(argListener);
	javax.swing.table.TableColumn tc =
	    argTable.getColumnModel().getColumn(0);
	tc.setPreferredWidth(50);
	tc.setHeaderValue("Name");
	tc = argTable.getColumnModel().getColumn(1);
	tc.setPreferredWidth(300);
	tc.setHeaderValue("Value");
	JScrollPane jsp = new JScrollPane(argTable);
	argListPanel.add(jsp,BorderLayout.CENTER);

  //Buttons for adding, editin, and deleting arguments
  JButton add = new JButton("Add");
  JButton edit = new JButton("Edit");
  JButton delete = new JButton("Delete");
  Dimension d = delete.getPreferredSize();
  add.setPreferredSize(d);
  edit.setPreferredSize(d);
  //action listeners for buttons
  add.addActionListener(argListener);
  edit.addActionListener(argListener);
  delete.addActionListener(argListener);

  //panel to hold these buttons
  JPanel buttonHolder = new JPanel(new FlowLayout(FlowLayout.CENTER,0,0));
  buttonHolder.add(add);
  buttonHolder.add(edit);
  buttonHolder.add(delete);
  argListPanel.add(buttonHolder,BorderLayout.SOUTH);

	return argListPanel;
    }

  public void setMimeType(String mimeType)
  {
  }

  public void setFilePath(String fn)
  {
    setFile(fn);
  }

    /**
     * Set the available thread groups.
     * @param groups An array of thread group names
     **/
    public void setThreadGroups(String[] groups) {
	threadGroupField.setListData(groups);
    }

    /**
     * Set the selected thread indices
     * @param indices An array of ints containing the selected indices.
     **/
    public void setSelectedThreadIndices(int[] indices) {
	threadGroupField.setSelectedIndices(indices);
    }

    /**
     * Get the indices of the selected threads.
     * @return An array of ints containing the selected indices.
     **/
    public int[] getSelectedThreadIndices() {
	return threadGroupField.getSelectedIndices();
    }

    /**
     * Set the url of the sample GUI
     * @param url The url to set.
     **/
    public void setURL(String url) {
	urlField.setText(url);
    }

    /**
     * Is the sample a POST?
     **/
    public boolean isPost() {
	return postButton.isSelected();
    }

    /**
     * Set if the sample is a POST.
     **/
    public void setPost(boolean value) {
	postButton.setSelected(value);
    }

    /**
     * Is the sample a GET?
     **/
    public boolean isGet() {
	return getButton.isSelected();
    }

    /**
     * Set if the sample is a GET
     **/
    public void setGet(boolean value) {
	getButton.setSelected(value);
    }

    /**
     * Should the sample be sampled once?
     **/
    public boolean isOnce() {
	return onceOnly.isSelected();
    }

    /**
     * Set if the sample should be executed once only.
     **/
    public void setOnce(boolean value) {
	onceOnly.setSelected(value);
    }

    public boolean isInterleave()
    {
      return interleave.isSelected();
    }

    public void setInterleave(boolean value)
    {
      interleave.setSelected(value);
    }

    public void setFile(String file) {
    	fileList.removeItem(file);
    	fileList.addItem(file);
    	fileList.getEditor().setItem(file);
    }

    /** Listener for argument table.  */
    private class ArgumentListener extends KeyAdapter
	implements MouseListener, ActionListener {

	public void mouseClicked(MouseEvent e)
	{
	    if((e.getModifiers() == e.BUTTON3_MASK ||
		e.getModifiers() == e.BUTTON2_MASK)
	       && argTable.getSelectedColumn() == 1) {
		String[] values =
		    JMeterUtils.split((String)argTable.getValueAt
				      (argTable.getSelectedRow(),
				       argTable.getSelectedColumn()),
				      "\",\"","");
		for(int x = 0;x < values.length;x++) {
		    if(values[x].startsWith("\""))
			values[x] = values[x].substring(1);
		    if(values[x].endsWith("\""))
			values[x] =
			    values[x].substring(0,values[x].length()-1);
		}
		multList.setItems(values);
		multValues.getContentPane().removeAll();
		multValues.getContentPane().add(multList);
		multValues.show();
	    }
	}

	public void mouseReleased(MouseEvent e) {}
	public void mousePressed(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}

  private boolean adding = false;

	public void actionPerformed(ActionEvent e)
	{
	    String action = e.getActionCommand();
    JComponent c = (JComponent) e.getSource();
    if(c == fileList) {
      String file = (String) fileList.getSelectedItem();
      setFile(file);
    }
    else if (c == browseButton)
    {
  		File tmp = JMeterUtils.promptToOpenFile().getSelectedFile();
  		if( tmp != null )
  		    setFile(tmp.toString());
    }
	  else if(action.equals("OK"))
    {
  		String[] values = multList.getItems();
      if(adding == true)
      {
        //if currently selected row contains any values, then add a row
        if(argTable.getSelectedRow() == -1)
        {
          if(argTable.getRowCount() == 0)
            ((DefaultTableModel)argTable.getModel()).addRow
  		            (new String[] {"",""});
          argTable.setRowSelectionInterval(0,0);
        }
        String col0 = (String)argTable.getValueAt(argTable.getSelectedRow(),0);
        String col1 = (String)argTable.getValueAt(argTable.getSelectedRow(),1);
        if(!((col0 == null || col0.equals("")) &&
                 (col1 == null || col1.equals(""))))
        {
          ((DefaultTableModel)argTable.getModel()).addRow
  		            (new String[] {"",""});
  		    argTable.setRowSelectionInterval(argTable.getRowCount()-1,
  						    argTable.getRowCount()-1);
        }
        adding = false;
      }
      argTable.setValueAt(multList.getName(),argTable.getSelectedRow(),0);
      if(values.length >0) {
  		  argTable.setValueAt("\""+JMeterUtils.unsplit
  				      (values,"\",\"")+"\"",
  				      argTable.getSelectedRow(),1);
      }
  		    multValues.setVisible(false);
  		    multValues.dispose();
	    }
	    else if(action.equals("Cancel")) {
		multValues.setVisible(false);
		multValues.dispose();
	    }
      else if(action.equals("Edit")) {
        int col = 1;
        int row = argTable.getSelectedRow();
        String[] values =
		    JMeterUtils.split((String)argTable.getValueAt
				      (row,col),
				      "\",\"","");
		    for(int x = 0;x < values.length;x++) {
		      if(values[x].startsWith("\""))
			      values[x] = values[x].substring(1);
		      if(values[x].endsWith("\""))
			      values[x] = values[x].substring(0,values[x].length()-1);
		    }
		    multList.setItems(values);
        multList.setName((String)argTable.getValueAt(row,0));
    		multValues.getContentPane().removeAll();
    		multValues.getContentPane().add(multList);
    		multValues.show();
      }
      else if(action.equals("Add")) {
        adding = true;
        multList.setItems(new String[0]);
        multList.setName("");
    		multValues.getContentPane().removeAll();
    		multValues.getContentPane().add(multList);
    		multValues.show();
      }
      else if(action.equals("Delete")) {
        ((DefaultTableModel)argTable.getModel()).removeRow(argTable.getSelectedRow());
		  }
	}

	public void keyTyped(KeyEvent e)
	{
	    if ((e.getKeyChar()=='\n'||e.getKeyChar()=='\r') &&
		argTable.getSelectedRow()==0) {
		((DefaultTableModel)argTable.getModel()).addRow
		    (new String[] {"",""});
		argTable.setRowSelectionInterval(argTable.getRowCount()-1,
						 argTable.getRowCount()-1);
	    }
	}
    }
    /*   End Listener  */

    /**
     * Generate the default (empty) argument fields.
     **/
    public void generateArgumentFields()
    {
	generateArgumentFields(new String[] {""}, new String[] {""});
    }

    /**
     * Set the argument fields to the given values.
     **/
    public void generateArgumentFields(String[] names,String[] values)
    {
	String[][] args = new String[names.length][2];
	for(int i=0;i<names.length;i++)
	    args[i] = new String[] {names[i], values[i]};

	generateArgumentFields(args);
    }

    /**
     * Set the argument fields to the given values.
     **/
    public void generateArgumentFields(String[][] values)
    {
	DefaultTableModel dtm = (DefaultTableModel)argTable.getModel();
	while(dtm.getRowCount()>0)
	    dtm.removeRow(0);
	for(int i=0;i<values.length;i++)
	    dtm.addRow(new String[] {values[i][0],values[i][1]});
    }

    /**
     * Get the current URL
     **/
    public String getUrl()
    {
	return urlField.getText();
    }

    /**
     * Get the current arguments
     * @return a 2xn array of arguments {{name1,value1},...,{nameN, valueN}}
     **/
    public String[][] getArguments()
    {
	javax.swing.table.TableModel tm = argTable.getModel();
	Vector v = new Vector();
	for(int i=0;i<argTable.getRowCount();i++)
	    if (tm.getValueAt(i,0)!=null &&
		!"".equals(tm.getValueAt(i, 0)))
		v.addElement(new String[] {(String)tm.getValueAt(i, 0),
					   (String)tm.getValueAt(i, 1)});
	String[][] result = new String[v.size()][];
	v.copyInto(result);
	return result;
    }

    /**
     * Wipes out all previous argument fields.
     */
    private void clearArguments()
    {
	generateArgumentFields();
    }
}


