/*
 * Copyright (c) 1998-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * 4. The names "Apache JMeter" and "Java Apache Project" must
 *    not be used to endorse or promote products derived from this software
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JMeter"
 *    nor may "Java Apache Project" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */
package org.apache.jmeter.controllers;

import java.util.Vector;
import org.xml.sax.*;
import org.xml.sax.helpers.*;

/**
 * Provides a series of callbacks for the XML parser to convert a
 * saved HTTP Sample back into its Java representation.
 */
public class HTTPSampleDocumentHandler extends HandlerBase {
    private String url;
    private boolean once;
    private String method;
    private String name;
    private String className;
    private String[][] arguments;
    private String argName;
    private Vector args;
    private boolean inArgs;

    public HTTPSampleDocumentHandler() {
	args=new Vector();
    }

    /**
     * Get the TestSample URL
     **/
    public String getURL() {
	return url;
    }

    /**
     * Get the TestSample class
     **/
    public String getClassName()
    {
      return className;
    }

    /**
     * Get the TestSample Name
     **/
    public String getName()
    {
        System.out.println("Name of sample = "+name+" class="+this);
      return name;
    }

    /**
     * Should this sample be executed once?
     **/
    public boolean isOnce() {
	return once;
    }

    /**
     * Get the method of sampling (currently only GET and POST)
     **/
    public String getMethod() {
	return method;
    }

    /**
     * Get the URL arguments (query string)
     **/
    public String[][] getArguments() {
	return arguments;
    }

    /**
     * org.xml.sax.DocumentHandler implementation.
     **/
    public void startDocument() {
	args.removeAllElements();
	inArgs = false;
    }

    /**
     * org.xml.sax.DocumentHandler implementation.
     **/
    public void characters(char[] data,int start,int length)
    {
      String value = new String(data,start,length);
      if(argName != null)
        if(!value.trim().equals(""))
          args.add(new String[]{argName,value.trim()});
    }


    /**
     * org.xml.sax.DocumentHandler implementation.
     **/
    public void startElement(String name, AttributeList atts)
	throws SAXException
    {
      argName = null;
      if ("httpsample".equalsIgnoreCase(name))
      {
        url = atts.getValue("url");
	once = "true".equalsIgnoreCase(atts.getValue("once"));
        method = atts.getValue("method");
        className = atts.getValue("class");
        this.name = atts.getValue("name");
        System.out.println("Name of sample = "+this.name+" class="+this);
      }
      else if ("arguments".equalsIgnoreCase(name))
      {
        inArgs = true;
      }
      else if ("argument".equalsIgnoreCase(name))
      {
        if (!inArgs)
	  throw new SAXException("Argument tag not inside Arguments tag.");
        argName = atts.getValue("name");
      }
    }

    /**
     * org.xml.sax.DocumentHandler implementation.
     **/
    public void endElement(String name) {
	if ("arguments".equalsIgnoreCase(name))
	    inArgs = false;
    }

    /**
     * org.xml.sax.DocumentHandler implementation.
     **/
    public void endDocument() {
	    arguments = new String[args.size()][];
	    args.copyInto(arguments);
    }
}

