 /*
 * Copyright (c) 1998-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * 4. The names "Apache JMeter" and "Java Apache Project" must
 *    not be used to endorse or promote products derived from this software
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JMeter"
 *    nor may "Java Apache Project" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */
package org.apache.jmeter.controllers;
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

import org.xml.sax.*;

import org.apache.jmeter.JMeterUtils;
import org.apache.jmeter.threads.*;
import org.apache.jmeter.samplers.*;
import org.apache.jmeter.util.*;
/**
 * Controls the lifecycle of a HTTPSampler by taking care of the
 * configuration of the HTTPSampler,
 * the cookie manager (if needed) and
 * the authentication manager (if needed).
 */
public class HTTPSamplerController
    implements SamplerController, ThreadGroupListener, Saveable
{
    private static String[] defaultThreadGroups = {"Group 1"};
    /* Wouldn't it be better to use a Vector here,
     * since stuff is added and removed regularly?
     */
    private String[] threadGroups = defaultThreadGroups;
    private static String[] testSampleNameList;
    private Map tsNameClassMap = new HashMap();
    private String propName;
    private TestSample currentSample;
    boolean editing=false;
    TestSampleListModel testSampleModel = new TestSampleListModel();
    JFileChooser jfc;
    /* GUI variables */
    ControllerGUI gui;
    /**
     * Allows the user to select if they need cookie services,
     * and if so, specify parameters for it.
     */
    CookiePanel cookiePanel = new CookiePanel();
    /**
     * Allows the user to select if they need authorization services,
     *  and if so, specify parameters for it.
     */
    AuthPanel authPanel = new AuthPanel();
    JList entryList;
    JComboBox testSamples;
    /**
     * The pop-up for editing any TestSample parameters.
     */
    JDialog entryDialog;
    /**
     * The box for the OK, Cancel, (optionally) Load and Save buttons
     *  on the URL Test Parameters dialog.
     */
    Box buttonBox;
    JButton save;
    JButton load;
    JButton createTestSampleButton;
    JButton editTestSampleButton;
    JButton deleteTestSampleButton,saveButton,loadButton;
    public HTTPSamplerController()
    {
    }
    public String[] getDefaultThreadGroups() {
	return defaultThreadGroups;
    }

    public void setTestSamples(String[] ts) {

      testSampleNameList = ts;
      Class temp;
      for(int x = 0;x < ts.length;x++)
      {
        try{
          temp = Class.forName("org.apache.jmeter.controllers."+testSampleNameList[x]);
          testSampleNameList[x] = (String)temp.getMethod("getClassDescName",
                  new Class[0]).invoke(temp.newInstance(),new Object[0]);
          tsNameClassMap.put(testSampleNameList[x],temp);
        }catch(Exception e){e.printStackTrace();}
      }
    }
    public void start()
    {
	cookiePanel.disable();
	authPanel.disable();
	testSamples.setEnabled(false);
	createTestSampleButton.setEnabled(false);
	editTestSampleButton.setEnabled(false);
	deleteTestSampleButton.setEnabled(false);
      saveButton.setEnabled(false);
      loadButton.setEnabled(false);
    }
    public void stop()
    {
	cookiePanel.enable();
	authPanel.enable();
	testSamples.setEnabled(true);
	createTestSampleButton.setEnabled(true);
	editTestSampleButton.setEnabled(true);
	deleteTestSampleButton.setEnabled(true);
      saveButton.setEnabled(true);
      loadButton.setEnabled(true);
    }
    /*****
     * Thread Group Listener Interface
     *****/
    public void threadGroupAdded(ThreadGroupEvent e) {
	String[] temp = new String[threadGroups.length+1];
	int i;
	for(i=0;i<threadGroups.length;i++)
	    temp[i]=threadGroups[i];
	temp[i]=e.getThreadGroup();
	threadGroups=temp;
    }

    public void threadGroupRemoved(ThreadGroupEvent e) {
	Vector v = new Vector();
	for(int i=0;i<threadGroups.length;i++)
	    if (!threadGroups[i].equals(e.getThreadGroup()))
		v.addElement(threadGroups[i]);
	threadGroups = new String[v.size()];
	v.copyInto(threadGroups);
    }
    public void threadGroupSelected(ThreadGroupEvent e) {}
    /****
     * End ThreadGroupListener
     ****/
    public JMeterThread[] getSampleThreads(String threadGroup, int count) {
        TestSample[] samples = testSampleModel.getTestSamples(threadGroup);
	if (samples != null && samples.length > 0) {
	    JMeterThread[] result = new JMeterThread[count];
	    for (int i=0;i<result.length;i++)
      {
	  Sampler s = getSampler();
	  result[i]=new HTTPJMeterThread(s,samples,threadGroup);
      }
	    return result;
	}
	return null;
    }
    /**@link dependency*//*#HTTPSampler lnkHTTPSampler;*/
    public Sampler getSampler() {
      Sampler s;
      if(cookiePanel.getCookieMgr() == null)
        s = new HTTPSampler(null,authPanel.getAuthMgr());
      else
        s = new HTTPSampler((CookieManager)cookiePanel.getCookieMgr().clone(),
			       authPanel.getAuthMgr());
	    return s;
    }

    /**
     * Test samples are the obvious saveable objects that would be added to
     * a controller currently.  More will come in the future.
     */
    public void addSaveableObject(Saveable o)
    {
    }

    public void addTestSample(TestSample testSample)
    {
    }

  /**
   *  Gets the SAX document handler to read the xml that stores information
   *  necessary for the reconstruction of this type of object.
   */
  public HandlerBase getHandlerBase()
  {
    return (HandlerBase)JMeterUtils.getXMLDocumentHandler("httpcontroller");
  }

    /**@link dependency*//*#HTTPSampleDocumentHandler hnd;*/
  public void load(HandlerBase agent)
  {
    /**
     * Here's the implemention once I write the HTTPControllerDocumentHandler.
      It's incomplete and mostly just copied from HTTPBaseTestSample
    HTTPSampleDocumentHandler hnd;
	  try {
		  hnd = (HTTPControllerDocumentHandler)handler;
      if(hnd.getClassName().equals(this.getClass().getName()))
      {
		    // code to load data into controller from handler
      }
      else
        JOptionPane.showMessageDialog(null,"Saved Test Sample doesn't match this class",
              "Error",JOptionPane.ERROR_MESSAGE);
	  } catch (Exception e) {
        JOptionPane.showMessageDialog(null,"Error occurred while parsing XML file",
              "Error",JOptionPane.ERROR_MESSAGE);
		    e.printStackTrace();
	  }*/
  }

  public void save(OutputStream output) {
	PrintWriter out = new PrintWriter(output);

	out.println("<?xml version=\"1.0\" ?>");
	// code to output controller to xml
	out.flush();
    }

    public Container getControlPanel()
    {
	if(gui == null)
	    gui = new ControllerGUI();
	return gui;
    }
    public String getName()
    {
	return "Advanced HTTP Tester";
    }
    public String getPropName()
    {
      return propName;
    }
    public void setPropName(String p)
    {
      propName = p;
    }
    public String toString() {
	return getName();
    }

    private Box getButtonBox(boolean saveEnabled) {
	if (buttonBox==null) {
	    JButton ok = new JButton("OK");
	    JButton cancel = new JButton("Cancel");
	    save = new JButton("Save");
	    load = new JButton("Load");
	    buttonBox = new Box(BoxLayout.X_AXIS);
	    DialogButtonListener dbl = new DialogButtonListener();
      DialogSaveAndLoadListener dsll = null;
      try {
        dsll = new DialogSaveAndLoadListener();
      }catch(Throwable e){}
	    ok.addActionListener(dbl);
	    cancel.addActionListener(dbl);
	    save.addActionListener(dsll);
	    load.addActionListener(dsll);
	    buttonBox.add(ok);
	    buttonBox.add(cancel);
      if(dsll != null)
      {
	      buttonBox.add(save);
	      buttonBox.add(load);
      }
	}
	save.setEnabled(saveEnabled);
	load.setEnabled(saveEnabled);
	return buttonBox;
    }
    /**@link dependency*//*#TestSampleCellRenderer lnkTestSampleCellRenderer;*/
    private class ControllerGUI extends JPanel
    {
	public ControllerGUI()
	{
	    super();
	    TestSampleChooserListener tscl = null;
      SaveAndLoadListener sall = null;
        tscl = new TestSampleChooserListener();
      try{
        sall = new SaveAndLoadListener();
      }catch(Throwable e){e.printStackTrace();}
            testSamples = new JComboBox(testSampleNameList);
	    setLayout(new GridBagLayout());
	    JPanel jp = new JPanel();
	    JPanel subPanel = new JPanel();
    subPanel.add(cookiePanel);
	    subPanel.add(authPanel);
	    JPanel entryBox = new JPanel(new FlowLayout(FlowLayout.LEFT,10,1));
	    testSamples.setEditable(false);
	    createTestSampleButton = JMeterUtils.createButton("New24",tscl);
      createTestSampleButton.setToolTipText("Create new test sample");
      createTestSampleButton.setActionCommand("create");
      createTestSampleButton.setMargin(new Insets(0,0,0,0));
	    editTestSampleButton = JMeterUtils.createButton("Edit24",tscl);
      editTestSampleButton.setToolTipText("Edit test sample");
      editTestSampleButton.setActionCommand("edit");
      editTestSampleButton.setMargin(new Insets(0,0,0,0));
	    deleteTestSampleButton = JMeterUtils.createButton("Delete24",tscl);
      deleteTestSampleButton.setToolTipText("Delete test sample");
      deleteTestSampleButton.setActionCommand("delete");
      deleteTestSampleButton.setMargin(new Insets(0,0,0,0));
      saveButton = JMeterUtils.createButton("SaveAll24",sall);
      saveButton.setToolTipText("Save all test sampels");
      saveButton.setActionCommand("save");
      saveButton.setMargin(new Insets(0,0,0,0));
      loadButton = JMeterUtils.createButton("Open24",sall);
      loadButton.setToolTipText("Load test samples from file");
      loadButton.setActionCommand("load");
      loadButton.setMargin(new Insets(0,0,0,0));
	    entryBox.add(testSamples);
	    entryBox.add(createTestSampleButton);
      if(sall != null)
      {
        entryBox.add(loadButton);
        entryBox.add(saveButton);
      }
	    entryBox.add(editTestSampleButton);
	    entryBox.add(deleteTestSampleButton);
      entryBox.add(Box.createGlue());
	    GridBagConstraints gbc = new GridBagConstraints();
	    gbc.fill=gbc.BOTH;
	    gbc.insets = new Insets(3,1,1,3);
	    gbc.gridx = 0;
	    gbc.gridy = 0;
      gbc.weightx=1;
	    add(subPanel, gbc.clone());
      gbc.anchor=gbc.WEST;
	    gbc.gridy++;
	    add(entryBox, gbc.clone());
      gbc.fill=gbc.BOTH;
	    gbc.gridy++;
            gbc.weighty=5;
	    entryList = new JList(testSampleModel);
	    entryList.setCellRenderer(new TestSampleCellRenderer());
	    entryList.setVisibleRowCount(10);
	    add(new JScrollPane(entryList),gbc.clone());
	    validate();
	}
    }

    private class SaveAndLoadListener implements ActionListener
    {
      public void actionPerformed(ActionEvent e)
	    {
        String cmd = e.getActionCommand();
        if (cmd.equals("save"))
        {
  		    File tmpFile = JMeterUtils.promptToSaveFile().getSelectedFile();
          if(tmpFile != null)
          {
            try {
              saveAllTestSamples(tmpFile);
            }catch(IOException err)
            {
              JOptionPane.showMessageDialog(null,"You must choose a valid file name",
                  "Error",JOptionPane.ERROR_MESSAGE);
            }
          }
          else  // they hit cancel
		        return;
        }
	      else if (cmd.equals("load"))
        {
  			  File tmpFile = JMeterUtils.promptToOpenFile().getSelectedFile();
			    // They changed their mind and hit Cancel
			    if( tmpFile == null ) return;
          try {
    			  if( tmpFile.canRead() )
            {
              SaveableObjectHandler soh = new SaveableObjectHandler();
              soh.initialize();
              org.xml.sax.Parser parser = JMeterUtils.getXMLParser();
              parser.setDocumentHandler(soh);
              parser.parse(new org.xml.sax.InputSource(new FileInputStream(tmpFile)));
              Saveable[] savedTestSamples = soh.getSavedObjects();
              for(int x = 0;x < savedTestSamples.length;x++)
              {
                ((TestSample)savedTestSamples[x]).setThreadGroups(threadGroups);
                testSampleModel.addTestSample((TestSample)savedTestSamples[x]);
              }
    				  //((Saveable)ts).load(new FileInputStream(tmpFile));
    			  }
            else
            {
    				  JOptionPane.showMessageDialog(null,
    					        "The file you specified cannot be read.","Information",
    						      JOptionPane.INFORMATION_MESSAGE);
    				  return;
    			  }
  		    }catch (NoClassDefFoundError err) {
                  JOptionPane.showMessageDialog(null,"You must use a SAX 1.0 XML parser "+
                          "\nfor this to work.  Please see documentation "+
                          "\nfor further details","Information",JOptionPane.INFORMATION_MESSAGE);
		        err.printStackTrace();
		      }catch(Exception err){err.printStackTrace();}
	      }
      }
    }

    private class TestSampleChooserListener implements ActionListener
    {
	public void actionPerformed(ActionEvent e)
	{
	    String cmd = e.getActionCommand();
	    if (cmd.equals("create")) {
		editing=false;
		String className = (String)testSamples.getSelectedItem();
		TestSample ts = null;
		try{
		    ts = (TestSample)((Class)tsNameClassMap.get(className)).newInstance();
		}catch(Exception err){}
		ts.setThreadGroups(threadGroups);
		currentSample=ts;
		entryDialog =
		    new JDialog();
		entryDialog.setSize(400,500);
		entryDialog.getContentPane().setLayout(new BoxLayout
		    (entryDialog.getContentPane(),BoxLayout.Y_AXIS));
		entryDialog.getContentPane().add(ts.getGUI());
		entryDialog.getContentPane().add
		    (getButtonBox(ts instanceof Saveable));
		entryDialog.show();
	}
	    else if (cmd.equals("edit")) {
		editing=true;
		TestSample ts = (TestSample)entryList.getSelectedValue();
    currentSample = ts;
		if (ts!=null) {
		    ts.setThreadGroups(threadGroups);
        entryDialog = new JDialog();
		    entryDialog.setSize(400,500);
		    entryDialog.getContentPane().setLayout(new BoxLayout
		    (entryDialog.getContentPane(),BoxLayout.Y_AXIS));
		    entryDialog.getContentPane().add(ts.getGUI());
		    entryDialog.getContentPane().add(getButtonBox(ts instanceof Saveable));
		    entryDialog.show();
		}
	    }
	  else if (cmd.equals("delete")) {
      int[] indices;
		  if ((indices = entryList.getSelectedIndices()) != null)
        testSampleModel.removeTestSamples(indices);
	  }
  }
  }

    private class DialogSaveAndLoadListener implements ActionListener
    {
      public void actionPerformed(ActionEvent ev)
      {
        TestSample ts = currentSample;
	      String cmd = ev.getActionCommand();
        if (cmd.equals("Save"))
      {
  		  File tmpFile = JMeterUtils.promptToSaveFile().getSelectedFile();
        if(tmpFile != null)
        {
          try {
            ((Saveable)ts).save(new FileOutputStream(tmpFile));
          }catch(IOException e)
            {
              JOptionPane.showMessageDialog(null,"You must choose a valid file name",
                  "Error",JOptionPane.ERROR_MESSAGE);
            }
        }
        else  // they hit cancel
		      return;
      }
	    else if (cmd.equals("Load"))
      {
  			File tmpFile = JMeterUtils.promptToOpenFile().getSelectedFile();
			// They changed their mind and hit Cancel
			if( tmpFile == null ) return;
        try {
    			if( tmpFile.canRead() ) {
            HandlerBase hnd = ((Saveable)ts).getHandlerBase();
            Parser parser = JMeterUtils.getXMLParser();
            parser.setDocumentHandler(hnd);
            parser.parse(new InputSource(new FileInputStream(tmpFile)));
    				((Saveable)ts).load(hnd);
    			}
          else
          {
    				JOptionPane
    					.showMessageDialog(
    						null,
    						"The file you specified cannot be read.",
    						"Information",
    						JOptionPane.INFORMATION_MESSAGE);
    				return;
    			}
  		  }catch (NoClassDefFoundError e) {
                  JOptionPane.showMessageDialog(null,"You must use a SAX 1.0 XML parser "+
                          "\nfor this to work.  Please see documentation "+
                          "\nfor further details","Information",JOptionPane.INFORMATION_MESSAGE);
		        e.printStackTrace();
		      }catch(Exception e){e.printStackTrace();}
	      }
      }
    }

    private class DialogButtonListener implements ActionListener
    {
	public void actionPerformed(ActionEvent ev)
	{
	    TestSample ts = currentSample;
	    String cmd = ev.getActionCommand();
		    if(cmd.equals("OK")) {
		      if (ts.getThreadGroups().length>0) {
			      if (editing==false) {
			        testSampleModel.addTestSample(ts);
			      }
			      else
			        testSampleModel.updateTestSample(ts);
		      }
		      else {
            JOptionPane.showMessageDialog(null,"You must select a thread group!",
                  "Information",JOptionPane.INFORMATION_MESSAGE);
            editing = false;
			      return;
		      }
		    }
		    entryDialog.getContentPane().removeAll();
		    entryDialog.dispose();
	  }
  }

  private void saveAllTestSamples(File file) throws IOException
  {
    String fs = System.getProperty("file.separator");
    String filename = file.getPath();
    String dir = filename.substring(0,filename.lastIndexOf(fs))+fs+"testsamples"+
            fs;
    File f = new File(dir);
    f.mkdirs();
    TestSample[] tempSamples = testSampleModel.getTestSamples();
    PrintWriter out = new PrintWriter(new OutputStreamWriter(
            new BufferedOutputStream(new FileOutputStream(file))));
    out.println("<?xml version=\"1.0\" ?>");
    out.println("<HttpSamplerController>");

    for(int x = 0;x < tempSamples.length;x++)
    {
      String tempFile = dir+tempSamples[x].getName()+".xml";
      out.println("<"+SaveableObjectHandler.FILE+" "+SaveableObjectHandler.FILENAME+
            "=\""+tempFile+"\"/>");
      FileOutputStream tempOut = new FileOutputStream(tempFile);
      ((Saveable)tempSamples[x]).save(tempOut);
      tempOut.close();
    }
    out.println("</HttpSamplerController>");
    out.flush();
    out.close();
  }

  public static final String ONCE = "httpcontroller.once";
  public static final String THREADGROUPS = "httpcontroller.threadgroups";
  public static final String INTERLEAVE = "httpcontroller.interleave";
}

