/*
 * Copyright (c) 1998-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * 4. The names "Apache JMeter" and "Java Apache Project" must
 *    not be used to endorse or promote products derived from this software
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JMeter"
 *    nor may "Java Apache Project" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 *
 * @author $Author: mstover $
 * @version $Version: 1.0 $
 */
package org.apache.jmeter.controllers;

import javax.swing.JPanel;
import  java.io.*;
import org.apache.jmeter.samplers.Entry;
/**
 * Specifies the functionality common to all types of Samples in the system.
 * This includes the ability to get all the entries in a sample,
 * get the Container used to configure it,
 * get (and set) the thread groups it is part of,
 * and get (and set) it's name.
 */
public interface TestSample
{
    /**
     * Get the GUI for inputing data to this test sample.
     **/
    public java.awt.Container getGUI();

    /**
     * @associates <{org.apache.jmeter.controllers.Entry}>
     */
    public Entry[] getEntries();

    /**
     * Get the thread groups associated with this sample.
     **/
    public String[] getThreadGroups();

    /**
     * Set the thread groups associated with this sample.
     **/
    public void setThreadGroups(String[] threadGroups);

    /**
     * Get the name of this sample.
     **/
    public String getName();

    /**
     * Set the name of this sample.
     **/
    public void setName(String name);

    /**
     * Reset the sample.
     **/
    public void reset();

    public String getClassDescName();
}
