/*
 * Copyright (c) 1998-99 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * 4. The names "Apache JMeter" and "Java Apache Project" must
 *    not be used to endorse or promote products derived from this software
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JMeter"
 *    nor may "Java Apache Project" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */
package org.apache.jmeter.samplers;

import java.sql.*;

import org.apache.jmeter.samplers.jdbc.Data;
import org.apache.jmeter.samplers.jdbc.*;
import org.apache.jmeter.JMeterUtils;

/**

 * A sampler which understands JDBC database requests

 */

public class JDBCSampler implements Sampler {
    String username;
    String password;
    String connect;
    String driver;
    long start,end,time;

    //database connection pool manager
    DBConnectionManager manager = new DBConnectionManager();
    DBKey key;

    public JDBCSampler(String username, String password, String connect,
            String driver,int connections,int use) {
      key = manager.startDatabaseConnection(connect,username,password,driver,
              use,connections);
    }

    public JDBCSampler(String username, String password, String connect,
            String driver)
    {
      this(username,password,connect,driver,10,500);
    }

    public SampleResult sample(Entry e) {
      time = start = end = 0;
      SampleResult res = new SampleResult();
      Connection con = null;
      ResultSet rs = null;
      Statement stmt = null;
      Data data = new Data();
      res.putValue(Sampler.SAMPLE_LABEL,e.getValue(QUERY));
	    start = System.currentTimeMillis();
	try {
	    while((con = manager.getConnection(key)) == null)
              try{Thread.sleep(10);}catch(Exception err){}
	    stmt = con.createStatement();
	    // Execute database query
	    boolean retVal = stmt.execute(e.getValue(QUERY).toString());
	    // Based on query return value, get results
	    if (retVal) {
		rs = stmt.getResultSet();
		data = getDataFromResultSet(rs);
                rs.close();
	    } else {
		int updateCount = stmt.getUpdateCount();
	    }
	    stmt.close();
            manager.releaseConnection(con);
	} catch (Exception ex) {
          if(rs != null)
          {
            try{
              rs.close();
            }catch(SQLException err){rs = null;}
          }
          if(stmt != null)
          {
            try{
              stmt.close();
            }catch(SQLException err){stmt = null;}
          }
          manager.releaseConnection(con);
		ex.printStackTrace();
	}
 	// Calculate response time
	  end = System.currentTimeMillis();
   time += end - start;
	res.setTime(time);
        res.putValue(Sampler.TEXT_RESPONSE,data);
	return res;
    }

      /***********************************
  Gets a Data object from a ResultSet.
@param  rs ResultSet passed in from a database query
@return A Data object (com.stover.utils)
@throws java.sql.SQLException
  *************************************/
  private Data getDataFromResultSet(ResultSet rs) throws SQLException
  {
    ResultSetMetaData meta;
    Data data=new Data();
    meta=rs.getMetaData();
    int numColumns = meta.getColumnCount();
    String[] dbCols=new String[numColumns];
    for(int count = 1;count<=numColumns;count++)
    {
       dbCols[count-1]=meta.getColumnName(count);
      data.addHeader(dbCols[count-1]);
    }
    while(rs.next())
    {
      data.next();
      for(int count = 0;count<numColumns;count++)
      {
        Object o=rs.getObject(count+1);
        end = System.currentTimeMillis();
        time += end - start;
        data.addColumnValue(dbCols[count],o);
        start = System.currentTimeMillis();
      }
    }
    return data;
  }// end method

  public final static String QUERY = "JDBCSampler.query";
}

