/*
 * Copyright (c) 1998-99 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * 4. The names "Apache JMeter" and "Java Apache Project" must
 *    not be used to endorse or promote products derived from this software
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JMeter"
 *    nor may "Java Apache Project" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 * @author $Author: mstover $
 * @version $Version: 1.0 $
 */
package org.apache.jmeter.threads;

import java.util.*;
import org.apache.jmeter.controllers.*;
import org.apache.jmeter.samplers.*;

/**
 * Provides the HTTP specific runtime ability for JMeter to
 * handle HTTP entries.
 */
public class HTTPJMeterThread extends JMeterThreadAdapter
{
  public HTTPJMeterThread(Sampler sampler, TestSample[] entries)
    {
	super(sampler,entries);
    }

    public HTTPJMeterThread(Sampler sampler, TestSample[] entries, String t)
    {
	super(sampler, entries,t);
    }

    public HTTPJMeterThread(Sampler sampler, TestSample[] entries,
			String t,Vector listeners)
    {
	super(sampler, entries, t,listeners);
    }

  public void run() {
	int index=0;
        int[] place = new int[samples.length];
        for(int x = 0;x < place.length;place[x++] = 0)
          samples[x].reset();
  	SampleResult res;
        Entry[] entries;
	timer.set();
	while(run && samples.length > 0) {
	    entries = samples[index].getEntries();
	    if(((Boolean)entries[0].getValue
		(HTTPSamplerController.ONCE)).booleanValue()) {
		if(((Boolean)entries[0].getValue
		    (HTTPSamplerController.INTERLEAVE)).booleanValue()) {
		    res = sample(entries[place[index]], threadGroup);
		    place[index] = (place[index]+1) % entries.length;
		    if(place[index] == 0) {
			samples = pullOutOfArray(samples,index);
			place = pullOutOfArray(place,index);
			index--;
		    }
		}
		else {
		    for(int x = 0;x < entries.length && run;x++)
			res = sample(entries[x], threadGroup);
		    samples = pullOutOfArray(samples,index);
		    place = pullOutOfArray(place,index);
		    index--;
		}
	    }
	    else {
		if(((Boolean)entries[0].getValue
		    (HTTPSamplerController.INTERLEAVE)).booleanValue()) {
		    res = sample(entries[place[index]], threadGroup);
		    place[index] = (place[index]+1) % entries.length;
		}
		else {
		    for(int x = 0;x < entries.length && run;x++)
			res = sample(entries[x], threadGroup);
		}
	    }
	    if (samples.length==0)
		break;
	    index=(index+1)%samples.length;

	    Thread.currentThread().yield();
	}
  }



    /**
    * Pulls out one item from an array, where count is the index of the item to be
    * pulled and rearranges the rest.  Assumes the array is of type String
    */
    public static TestSample[] pullOutOfArray(TestSample[] array, int count)
    {
	int length=array.length;
        TestSample[] new_array=new TestSample[length-1];
	int counter=0;
	while(counter<count)
		new_array[counter]=array[counter++];
	while (count<(length-1))
	{
		new_array[count]=array[count+1];
		count++;
	}


	return new_array;
    }

    /**
     * Pulls out one item from an array, where count is the index of the item to be
     * pulled and rearranges the rest.  Assumes the array is of type String
     */
    public static int[] pullOutOfArray(int[] array, int count)
    {
	int length=array.length;
        int[] new_array=new int[length-1];
	int counter=0;
	while(counter<count)
		new_array[counter]=array[counter++];
	while (count<(length-1))
	{
		new_array[count]=array[count+1];
		count++;
	}


	return new_array;
    }
}
